//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file geometry/olap/include/OlapGenerator.hh
/// \brief Definition of the OlapGenerator class
//
//
// $Id$
//
// 
// --------------------------------------------------------------
// OlapGenerator
//
// Author: Martin Liendl - Martin.Liendl@cern.ch
//
// --------------------------------------------------------------
//
#ifndef OlapGenerator_h
#define OlapGenerator_h

#include <vector>
#include "globals.hh"
#include "G4VUserPrimaryGeneratorAction.hh"
#include "G4ThreeVector.hh"

class G4VisExtent;

class OlapGrid 
{
public:
   OlapGrid();
   void Reset() { for (G4int i=0;i<3;i++) count[i]=0; axis=0; }
   void Next();
   std::vector<G4int> count; // 
   std::vector<G4int> gsize; // gridsize
   G4int axis, eventsPerRun;
};


class OlapGenerator : public  G4VUserPrimaryGeneratorAction
{

public:
   OlapGenerator();
   ~OlapGenerator();

   void GeneratePrimaries(G4Event*);
   
   void SetExtent(const G4VisExtent&);
   
   void SetExtent(G4double);
   
   void SetAutoIncrement(G4bool b) { autoinc = b; }
   
   void SetGrid(G4int,G4int,G4int);
   
   void Reset();
   
   G4int GetAxis() { return grid.axis; };
   
   OlapGrid grid; // basically a vector of <int>, size==3
   std::vector<G4double> ext;
   G4ThreeVector posAB, posBA;
   G4bool autoinc;
};
#endif
