//
// ********************************************************************
// * License and Disclaimer                                           *
// *                                                                  *
// * The  Geant4 software  is  copyright of the Copyright Holders  of *
// * the Geant4 Collaboration.  It is provided  under  the terms  and *
// * conditions of the Geant4 Software License,  included in the file *
// * LICENSE and available at  http://cern.ch/geant4/license .  These *
// * include a list of copyright holders.                             *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.  Please see the license in the file  LICENSE  and URL above *
// * for the full disclaimer and the limitation of liability.         *
// *                                                                  *
// * This  code  implementation is the result of  the  scientific and *
// * technical work of the GEANT4 collaboration.                      *
// * By using,  copying,  modifying or  distributing the software (or *
// * any work based  on the software)  you  agree  to acknowledge its *
// * use  in  resulting  scientific  publications,  and indicate your *
// * acceptance of all terms of the Geant4 Software license.          *
// ********************************************************************
//
/// \file geometry/olap/include/OlapNotify.hh
/// \brief Definition of the OlapNotify class
//
//
// $Id$
//
// 
// --------------------------------------------------------------
// OlapNotify
//
// Register a subclass of OlapNotify with OlapManager to be
// notified when the geometry changes or when overlap are
// reported.
//
// Author: Martin Liendl - Martin.Liendl@cern.ch
//
// --------------------------------------------------------------
//
#ifndef OlapNotify_h
#define OlapNotify_h

#include <vector>

#include "OlapEventAction.hh"

class G4LogicalVolume;

class OlapNotify
{
public:
  OlapNotify();
  virtual ~OlapNotify();
  
  virtual void worldChanged(G4LogicalVolume* newWorld) = 0;
  virtual void overlaps(const std::vector<OlapInfo*> &)=0;
                                  
};                                                                                                      
#endif
